/*
* style.css
* http://tasuke.dna.affrc.go.jp/
*
* Author: Jungsok Kim, Ryutaro Itoh
*
* Copyright (c) 2013 National Institute of Agrobiological Sciences   
*
* This software is released under the MIT License.
*
* http://opensource.org/licenses/mit-license.php
*/

body {
  margin: 0;
  padding: 0;  
}

progress {
  width: 250px;
  height: 20px;
}

#tasuke {
  position: absolute;
  top: 0;
  width: 100%;
  min-width : 1100px;
}

#top {
  background-color: #006666;
  border-top: 1px solid gray;
  color: white;
}

#menu {
  margin-top: 10px;
  margin-left: 10px;
  position: relative;
  z-index: 1;
  /*font-family: Verdana, Arial, sans-serif;*/
}

#track {
  clear: both;
  overflow:visible;
  margin-top: 5px;
  margin-left: 20px;
}

#trackRefName{
  text-align: center;
  vertical-align: top;
  margin-bottom: 5px;
}

#trackRefName p{
  margin:5px;
}

#content {
  overflow-y:auto;
  overflow-x:hidden;
  margin-left: 20px;
  min-width : 1100px;
}

#content2 {
  position: relative;
  margin-left: 20px;
}

#bottom {
  position: relative;
  background-color: #006666;
  color: white;
  z-index: 0;
}

#indicator{ 
  margin-left: 20px;
  height: 1px;
}

#ind { 
  position: absolute; 
  margin-top: -150px;
  margin-left: -30px;
  z-index: 1;
}

#zoombar{
  height: 1px;
  margin-bottom: 18px;
  position: relative;
  z-index: 2;
}

#zbar{
  margin-top: -30px;
  position: absolute;
}

#cellinf{
  position: absolute; 
  z-index: 2;
  top : 0px;
  left : 50px;
}

#dialog_color_manager  input {
  font-family: Courier New;
}

#cellinf_doc{
  font-family: Courier New;
  font-size:  12px;
  background-color: white;
}

.ui-widget-overlay {
  background: #000;
}

.cellDigInfoTd {
  table-layout: fixed;
  word-wrap: break-word;
  word-break: break-all;
}

.cellTdBlank { 
  border-style:none; 
}

.curHelp{
  cursor:help; 
}

.curPointer{
  cursor:pointer; 
}

#dialog_p3 .ui-widget-header {
  border: 0;
}

.fontCourierNew{
  font-family: Courier New;
  table-layout: fixed;
  word-wrap: break-all;
  word-break: break-all;
}

.colorpicker, .colorpicker * {
  z-index: 9990;
}

.colorForm{
  width:60px;
}

.gffWindowItems{
  padding-left: 20px;
  color:black;
  text-decoration: underline;
}

.gffWindowStr{
  padding-left: 20px;
  color:black;
}

.gffWindowItems a{
  color:#0000de;
}

.gffWindowItems:hover a{
  cursor:pointer;
  color:orange;
}

.gffWindowItems:hover{
  cursor:pointer;
  color:orange;
}

.gffWindowClose:hover{
  cursor:pointer;
}



